#include <LEDA/basic.h>
#include <LEDA/polygon.h>
#include <LEDA/gen_polygon.h>
#include <LEDA/window.h>
#include <LEDA/geowin.h>
#include <fstream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <iostream.h>
#include "Ray_crossings.h"
#include "Winding_number.h"
#include "Intersection.h"

using namespace leda;

static int x_max = 700 ;
static int y_max = 700 ;
static window W(x_max, y_max,"point-location & intersection");
Ray_crossings demo1;
Winding_number demo2;
int rek ;
int ecken;
float time1, time2, difference;
polygon P, Q;
int x;
list<point> generate_points(int n) // generate random points and store in point - list 
{
   list<point> L;
   point q;
   for (int i = 0 ; i < n ; i++)
   {
      point q ((rand() % 100), (rand() % 100));
      L.push_back(q);
   }
   return L;
}


polygon create_polygon() //read and draw polygon
{
   W.clear();
   W.set_color(LEDA::black);
   Q = W.read_polygon();
   W.draw_filled_polygon(Q,grey2);
   segment s;
   forall_segments(s,Q)
   {
      point a = s.start() ;
      point b = s.end() ;
      W.set_color(LEDA::black);
      W.draw_segment(a.xcoord(),a.ycoord(),b.xcoord(),b.ycoord());
   }
   return Q;
}

void show_panel() {
 panel Panel4;
 Panel4.text_item("");
 Panel4.text_item("\\bf Number of test-points:    ");
 Panel4.text_item("");
 Panel4.int_item("", x, 1,10,1);
 Panel4.int_item("", x, 100,1000,100);
 Panel4.int_item("", x, 10000,100000,10000);
 Panel4.open(W); 
}

void redraw() { }


int main()
{
   W.set_redraw(redraw);

   //define menu
   menu point_location_menu;
   menu intersection_menu;
   menu polygon_menu;
   
   //Buttons 
   W.button("'Point in Polygon'",  polygon_menu);
   W.button("'Intersection'",  intersection_menu);
   
   //create sub-menus
   polygon_menu.button("test ray crossings on userdefined polygon ",1);
   polygon_menu.button("test winding number on userdefined polygon ",2);
   polygon_menu.separator();
   polygon_menu.button("test ray crossings on hilbert-polygon",3);
   polygon_menu.button("test winding number on hilbert-polygon",4);
   polygon_menu.separator();
   polygon_menu.button("test ray crossings on n-gon",5);
   polygon_menu.button("test winding number on n-gon",6);
   polygon_menu.separator();
   polygon_menu.button("compare time: ray crossings vs. winding number",7);
   intersection_menu.button("intersection of userdefined polygons",8);
   intersection_menu.button("intersection of rotated n-gons",9);
     
   // create menu-bar
   W.make_menu_bar();
   
   // display window
   W.display();

   bool check;
   int but;
   
   while ((but = W.read_mouse()) != 0)
   {
      switch (but)
      {
         case 1: //test ray crossings one userdefined polygon
         {  
            W.clear();
            show_panel();
            list<point> random_points = generate_points(x);
            list<point>::iterator it = random_points.end();

            P = create_polygon();
            
            time1 = used_time();
            while (++it != random_points.end())
            {
               check = demo1.ray_crossings(P,*it);
               if (check==true) W.set_color(LEDA::green);
               else W.set_color(LEDA::red);
               W << *it;
            }
            time2 = used_time();
            difference = time2-time1;
            cout << "\ntime in seconds:\t" << difference;     
            break;
	     }
	     
	     case 2: //test winding number on userdefined polygon
         {
            W.clear();
            show_panel();
            list<point> random_points = generate_points(x);
            list<point>::iterator it = random_points.end();

	        P = create_polygon();
            
            time1 = used_time();
            while (++it != random_points.end())
            {
               check = demo2.winding_number(P,*it);
               if (check==true) W.set_color(LEDA::green);
               else W.set_color(LEDA::red);
               W << *it;
            }
            time2 = used_time();
            difference = time2-time1;
            cout << "\ntime in seconds:\t" << difference;     
	        break;
         }
	  
	     case 3:  //test ray crossings on hilbert-polygon
	     { 
            W.clear();
            show_panel();  
            W.clear();      
            panel Panel;
            Panel.text_item("\\bf Hilbert curve");
            Panel.text_item("");
            Panel.text_item("\\bf Please chose depth of recursion:    ");
            Panel.text_item("");
            Panel.int_item("", rek, 3,7,1);
            Panel.open(W);
            W.clear();
	        W.set_color(LEDA::black);
	        polygon H = hilbert(rek, 1, 1, 93, 93);
	        W.draw_filled_polygon(H,grey1);
	        W << H ;
            list<point> random_points = generate_points(x);
            list<point>::iterator it = random_points.end();
            while (++it != random_points.end())
            {
               check = demo1.ray_crossings(H,*it);
               if (check==true) W.set_color(LEDA::green);
               else W.set_color(LEDA::red);
               W << *it;
            }
	        break; 
	     }
                
         case 4: //test winding-number on hilbert polygon
	     {  
            W.clear();
            show_panel();
            W.clear();
            panel Panel;
            Panel.text_item("\\bf Hilbert curve");
            Panel.text_item("");
            Panel.text_item("\\bf Please chose depth of recursion:    ");
            Panel.text_item("");
            Panel.int_item("", rek, 3,7,1);
            Panel.open(W);
            W.clear();
	        W.set_color(LEDA::black);
	        polygon H = hilbert(rek, 1, 1, 93, 93);
	        W.draw_filled_polygon(H,grey1);
	        W << H ;
            list<point> random_points = generate_points(x);
            list<point>::iterator it = random_points.end();
            while (++it != random_points.end())
            {
               check = demo2.winding_number(H,*it);
               if (check==true) W.set_color(LEDA::green);
               else W.set_color(LEDA::red);
               W << *it;
            }
	        break;
	     }
	     
         case 5:  //test ray-crossings on n-gon
	     {  
            W.clear();
            show_panel();
            W.clear();
            panel Panel2;
            Panel2.text_item("\\bf n-gons");
            Panel2.text_item("");
            Panel2.text_item("\\bf Please chose the number of vertices:    ");
            Panel2.text_item("");
            Panel2.int_item("", ecken, 10,100,10);
            Panel2.int_item("", ecken, 100,1000,100);
            Panel2.int_item("", ecken, 1000,10000,1000);
            Panel2.open(W);
            W.clear();
	        W.set_color(LEDA::black);
	        circle C(50,50,30);
            polygon N = n_gon(ecken, C, 1.0);
	        W.draw_filled_polygon(N,grey1);
	        W << N ;
            list<point> random_points = generate_points(x);
            list<point>::iterator it = random_points.end();
            while (++it != random_points.end())
            {
	           check = demo1.ray_crossings(N,*it);
               if (check==true) W.set_color(LEDA::green);
               else W.set_color(LEDA::red);
               W << *it;
            }  
            break;
	     }
	     
         case 6:  //test winding number on n-gon
	     {   
            W.clear();
            show_panel();
            W.clear();
            panel Panel2;
            Panel2.text_item("\\bf n-gons");
            Panel2.text_item("");
            Panel2.text_item("\\bf Please chose the number of vertices:    ");
            Panel2.text_item("");
            Panel2.int_item("", ecken, 10,100,10);
            Panel2.int_item("", ecken, 100,1000,100);
            Panel2.int_item("", ecken, 1000,10000,1000);
            Panel2.open(W);
            W.clear();
	        W.set_color(LEDA::black);
	        circle C(50,50,30);
            polygon N = n_gon(ecken, C, 1.0);
	        W.draw_filled_polygon(N,grey1);
	        W << N ;
            list<point> random_points = generate_points(x);
            list<point>::iterator it = random_points.end();
            while (++it != random_points.end())
            {
	           check = demo2.winding_number(N,*it);
               if (check==true) W.set_color(LEDA::green);
               else W.set_color(LEDA::red);
               W << *it;
            }    
            break;
	     }
  
         case 7: //compare time: ray crossings vs winding number
         {
            char *filename = "times.txt";
            std::ofstream ofl(filename);
            cout << "\n\n*** wait while writing results to file times.txt ***";
            list<point> random_points = generate_points(1000);
            ofl << "All tests are based on 1000 test points!";
            
            //HILBERT-TESTS
            int counter=0;
            float durchschnitt1=0;
            float durchschnitt2=0;            
            for (int i=1; i <=8; i++) 
            {
               counter++;
               polygon H = hilbert(i, 1, 1,93, 93);
               ofl << "\nHilbert depth '" << i << "' in sec:";
               ofl << "\tRC=";
               list<point>::iterator test = random_points.end();

               time1 = used_time();
               while (++test != random_points.end())
               {
                  check = demo1.ray_crossings(H,*test);
               }   
               time2 = used_time();
               difference = time2-time1;
               ofl << difference;
               durchschnitt1 = durchschnitt1 + difference;
               
               test = random_points.end();               
               time1 = used_time();
               while (++test != random_points.end())
               {
                  check = demo2.winding_number(H,*test);
               }                  
               time2 = used_time();
               difference = time2-time1;
               ofl << "\tWN=" << difference;
               durchschnitt2 = durchschnitt2 + difference;               
            }//end for 
            ofl << "\nAverage RC=" << durchschnitt1/counter;
            ofl << "\nAverage WN=" << durchschnitt2/counter;                           
            ofl << "\n\n";
                  
            //NGON-TESTS
            counter=0;
            for (int i=10; i <=1000; i=i+10) 
            {
               counter++;         
               circle C(50,50,30);
               polygon N = n_gon(1000, C, 1.0);
               ofl << "\nn-gon depth '" << i << "' in sec:";
               ofl << "\tRC=";
               list<point>::iterator test = random_points.end();

               time1 = used_time();
               while (++test != random_points.end())
               {
                  check = demo1.ray_crossings(N,*test);
               }   
               time2 = used_time();
               difference = time2-time1;
               ofl << difference;
               durchschnitt1 = durchschnitt1 + difference;               
               
               test = random_points.end();               
               time1 = used_time();
               while (++test != random_points.end())
               {
                  check = demo2.winding_number(N,*test);
               }                  
               time2 = used_time();
               difference = time2-time1;
               ofl << "\tWN=" << difference;               
               durchschnitt2 = durchschnitt2 + difference;               
            }//end for
            ofl << "\nAverage RC=" << durchschnitt1/counter;
            ofl << "\nAverage WN=" << durchschnitt2/counter;            
            ofl.close();
            cout << "\n*** FINISHED ***\n\n\n\n";
            break;
         }
            
         case 8: //intersection of userdefined polygons
	     {
	        W.clear();
	        W.set_color(LEDA::black);	 

	        bool conv = false;
	        while (conv == false) {
	           P = W.read_polygon();
               W.draw_filled_polygon(P,grey2);
	           W << P ;
	           if ((P.is_convex() == true) || (((P.complement()).is_convex()) == true)) { conv = true; }
	           else { W.message("NICHT KONVEX! BITTE NEU EINGEBEN!");
                      LEDA::wait(2);
                      W.del_message();
                      W.clear();
               }
            }
	        
	        conv = false;
	        while (conv == false) {
	           Q = W.read_polygon();
   	           W.draw_filled_polygon(Q,grey1);
               W << Q;
	           if ((Q.is_convex() == true) || (((Q.complement()).is_convex()) == true)) { conv = true; }
	           else { W.message("NICHT KONVEX! BITTE NEU EINGEBEN!");
                      LEDA::wait(2);
                      W.del_message();
                      W.clear();
                      W.draw_filled_polygon(P,grey2);
                      W << P ; 
               }    
            }    
	        
	        Intersection schnitt;	        
            list<point> schnittpunkte = schnitt.polygon_intersection(P,Q);
            polygon result(schnittpunkte);
            W.draw_filled_polygon(result,red);
	        W << result; 
            
            list<point>::iterator it = schnittpunkte.end();
            while (++it != schnittpunkte.end())
            {
               W.set_color(LEDA::red);
               W << *it;           
            }
            break;          
	     }
        
	     case 9: //intersection of rotated n-gones
	     {
            panel Panel3;
            Panel3.text_item("\\bf n-gons");
            Panel3.text_item("");
            Panel3.text_item("\\bf Number of n-gon-vertices:    ");
            Panel3.text_item("");
            Panel3.int_item("", ecken, 3,19,1);
            Panel3.int_item("", ecken, 20,100,10);
            Panel3.int_item("", ecken, 200,1000,100);
            Panel3.open(W);
	        W.clear();
	        W.set_color(LEDA::black);	 
   	        
            point center(50,50);
   	        circle C(50,50,35);
            P = n_gon(ecken, C, 0.1);
            P = P.rotate(center,1.0);
            W.draw_filled_polygon(P,grey2);
	        W << P ;
	        
            Q = P.rotate(center,3.3);
	        W.draw_filled_polygon(Q,grey1);
	        W << Q; 
	        
	        Intersection schnitt;	        
            list<point> schnittpunkte = schnitt.polygon_intersection(P,Q);
            polygon result(schnittpunkte);
            W.draw_filled_polygon(result,red);
	        W << result; 
            
            list<point>::iterator it = schnittpunkte.end();
            while (++it != schnittpunkte.end())
            {
               W.set_color(LEDA::red);
               W << *it;           
            }            
            break;
	     }    

      }//switch  
   }//while  

   return 0;
} // main

